package com.duben.roseplaylet.manager;

import android.content.Context;
import android.text.TextUtils;

import com.duben.roseplaylet.MintsApplication;
import com.duben.roseplaylet.common.AppConfig;
import com.duben.roseplaylet.utils.LogUtil;
import com.tencent.imsdk.BaseConstants;
import com.tencent.imsdk.v2.V2TIMConversationListener;
import com.tencent.imsdk.v2.V2TIMManager;
import com.tencent.imsdk.v2.V2TIMSDKListener;
import com.tencent.qcloud.tuicore.TUILogin;
import com.tencent.qcloud.tuicore.interfaces.TUICallback;
import com.tencent.qcloud.tuicore.interfaces.TUILoginConfig;
import com.tencent.qcloud.tuikit.timcommon.BuildConfig;
import com.tencent.qcloud.tuikit.timcommon.util.ThreadUtils;

import java.util.ArrayList;
import java.util.List;

public class LoginWrapper {
    private static final String TAG = LoginWrapper.class.getSimpleName();

    private static class LoginWrapperHolder {
        private static final LoginWrapper loginInstance = new LoginWrapper();
    }

    public static LoginWrapper getInstance() {
        return LoginWrapperHolder.loginInstance;
    }

    private V2TIMSDKListener v2TIMSDKListener;
    private V2TIMConversationListener v2TIMConversationListener;
    private List<AppLoginListener> appLoginObservers = new ArrayList<>();

    private LoginWrapper() {
        initIMSDKObserver();
    }

    private void initIMSDKObserver() {
        if (v2TIMSDKListener == null) {
            v2TIMSDKListener = new V2TIMSDKListener() {
                @Override
                public void onConnecting() {
                    for (AppLoginListener listener : appLoginObservers) {
                        listener.onConnecting();
                    }
                }

                @Override
                public void onConnectSuccess() {
                    for (AppLoginListener listener : appLoginObservers) {
                        listener.onConnectSuccess();
                    }

                    tryToAutoLogin();
                }

                @Override
                public void onConnectFailed(int code, String error) {
                    for (AppLoginListener listener : appLoginObservers) {
                        listener.onConnectFailed(code, error);
                    }
                }

                @Override
                public void onUserSigExpired() {
                    for (AppLoginListener listener : appLoginObservers) {
                        listener.onUserSigExpired();
                    }
                }
            };
        }

        if (v2TIMConversationListener == null) {
            v2TIMConversationListener = new V2TIMConversationListener() {
                @Override
                public void onSyncServerStart() {
                    for (AppLoginListener listener : appLoginObservers) {
                        listener.onSyncServerStart();
                    }
                }

                @Override
                public void onSyncServerFinish() {
                    for (AppLoginListener listener : appLoginObservers) {
                        listener.onSyncServerFinish();
                    }
                }

                @Override
                public void onSyncServerFailed() {
                    for (AppLoginListener listener : appLoginObservers) {
                        listener.onSyncServerFailed();
                    }
                }
            };
        }

        V2TIMManager.getInstance().addIMSDKListener(v2TIMSDKListener);
        V2TIMManager.getConversationManager().addConversationListener(v2TIMConversationListener);
    }

    public void addAppLoginObserver(final AppLoginListener observer) {
        if (observer == null) {
            return;
        }

        ThreadUtils.runOnUiThread(() -> {
            if (!appLoginObservers.contains(observer)) {
                appLoginObservers.add(observer);
            }
        });
    }

    public void removeLoginObserver(AppLoginListener observer) {
        if (observer == null) {
            return;
        }

        ThreadUtils.runOnUiThread(() -> appLoginObservers.remove(observer));
    }

    public void loginIMSDK(Context context, int sdkAppID, String userID, String userSig, final TUILoginConfig config, TUICallback tuiCallback) {
        TUILogin.login(context, sdkAppID, userID, userSig, config, new TUICallback() {
            @Override
            public void onError(final int code, final String desc) {
                UserManager.getInstance().setLastImLoginCode(code);
                if (tuiCallback != null) {
                    tuiCallback.onError(code, desc);
                }
            }

            @Override
            public void onSuccess() {
                if (config == null || false == config.isInitLocalStorageOnly()) {
                    UserManager.getInstance().setLastImLoginCode(BaseConstants.ERR_SUCC);
                }

                setIMSetting();


                if (tuiCallback != null) {
                    tuiCallback.onSuccess();
                }
            }
        });
    }

    private void tryToAutoLogin() {
        int loginStatus = V2TIMManager.getInstance().getLoginStatus();
        UserManager userInfo = UserManager.getInstance();
        int lastLoginCode = userInfo.getLastImLoginCode();

        if (loginStatus == V2TIMManager.V2TIM_STATUS_LOGOUT && !TextUtils.isEmpty(userInfo.getUserID())) {
            if (lastLoginCode >= BaseConstants.ERR_SDK_NET_ENCODE_FAILED && lastLoginCode <= BaseConstants.ERR_SDK_NET_SEND_REMAINING_TIMEOUT_NO_NETWORK) {
                LogUtil.i(TAG, "onConnectSuccess, login IMSDK");
                loginIMSDK(MintsApplication.getContext(), AppConfig.IM_SDK_APPID, userInfo.getUserID(),
                        GenerateTestUserSig.genTestUserSig(userInfo.getUserID()), getLoginConfig(),
                        new TUICallback() {
                            @Override
                            public void onSuccess() {
                                // do nothing
                            }

                            @Override
                            public void onError(int errorCode, String errorMessage) {
                                UserManager.getInstance().setLastImLoginCode(errorCode);
                                LogUtil.e(TAG, "tryToAutoLogin error:" + errorCode);
                                if (errorCode < BaseConstants.ERR_SDK_NET_ENCODE_FAILED
                                        || errorCode > BaseConstants.ERR_SDK_NET_SEND_REMAINING_TIMEOUT_NO_NETWORK) {

                                }
                            }
                        });
            }
        }
    }

    public abstract static class AppLoginListener {
        public void onConnecting() {
        }

        public void onConnectSuccess() {
        }

        public void onConnectFailed(int code, String error) {
        }

        public void onUserSigExpired() {
        }

        public void onSyncServerStart() {
        }

        public void onSyncServerFinish() {
        }

        public void onSyncServerFailed() {
        }
    }

    private void setIMSetting() {
        IMHelper.Companion.getInstance().setCustomTopView(null);
    }

    public static TUILoginConfig getLoginConfig() {
        TUILoginConfig config = new TUILoginConfig();
        if (BuildConfig.DEBUG) {
            config.setLogLevel(TUILoginConfig.TUI_LOG_DEBUG);
        }
        return config;
    }
}
