package com.duben.roseplaylet.manager

import android.content.Context
import com.alibaba.sdk.android.oss.*
import com.alibaba.sdk.android.oss.callback.OSSCompletedCallback
import com.alibaba.sdk.android.oss.common.OSSConstants
import com.alibaba.sdk.android.oss.common.OSSLog
import com.alibaba.sdk.android.oss.common.auth.OSSCredentialProvider
import com.alibaba.sdk.android.oss.common.auth.OSSFederationCredentialProvider
import com.alibaba.sdk.android.oss.common.auth.OSSFederationToken
import com.alibaba.sdk.android.oss.common.utils.IOUtils
import com.alibaba.sdk.android.oss.internal.OSSAsyncTask
import com.alibaba.sdk.android.oss.model.OSSRequest
import com.alibaba.sdk.android.oss.model.PutObjectRequest
import com.alibaba.sdk.android.oss.model.PutObjectResult
import com.duben.roseplaylet.BuildConfig
import com.duben.roseplaylet.MintsApplication
import com.duben.roseplaylet.utils.TimeRender
import org.json.JSONObject
import java.io.File
import java.net.HttpURLConnection
import java.net.URL
import java.util.*

/**
 * 阿里云oss 上传图片
 */
internal class OssManager private constructor() {

    private var context: Context? = null
    private var oss: OSS? = null
    private var task: OSSAsyncTask<*>? = null
    private var ossUploadImgListener: OssUploadImgListener? = null

    companion object {
        private val TAG = OssManager::class.java.simpleName

        /* OSS配置*/
        const val OSS_NETWORK = "https://mints-sh.oss-cn-shanghai.aliyuncs.com/"
        const val OSS_ENDPOINT = "http://oss-cn-shanghai.aliyuncs.com"
        const val OSS_STS_SERVER_URL = BuildConfig.MainIp + "common/reqOssTstToken/roseplaylet"
        const val OSS_BUCKET_NAME = "mints-sh"
        const val OSS_ROOT_PATH = "pkg_roseplaylet/img/"
        const val OSS_IMG_TYPE = ".jpeg"
        private var _inst: OssManager? = null

        @get:Synchronized
        val instance: OssManager?
            get() = if (_inst != null) {
                _inst
            } else {
                _inst = OssManager()
                _inst
            }
    }

    init {
        init()
    }

    private fun init() {
        context = MintsApplication.getContext()

        // STS鉴权模式
        val credetialProvider: OSSCredentialProvider = object : OSSFederationCredentialProvider() {
            override fun getFederationToken(): OSSFederationToken? {
                try {
                    val stsUrl = URL(OSS_STS_SERVER_URL)
                    val conn = stsUrl.openConnection() as HttpURLConnection
                    val input = conn.inputStream
                    val jsonText =
                        IOUtils.readStreamAsString(input, OSSConstants.DEFAULT_CHARSET_NAME)
                    val jsonObjs = JSONObject(jsonText)
                    val jsonData = jsonObjs.getJSONObject("data")
                    val jsonCredentials = jsonData.getJSONObject("credentials")
                    val ak = jsonCredentials.getString("accessKeyId")
                    val sk = jsonCredentials.getString("accessKeySecret")
                    val token = jsonCredentials.getString("securityToken")
                    val expiration = jsonCredentials.getString("expiration")
                    return OSSFederationToken(ak, sk, token, expiration)
                } catch (e: Exception) {
                    e.printStackTrace()
                    if (ossUploadImgListener != null) {
                        ossUploadImgListener!!.uploadImgFailForSubthread("服务器开小差了")
                    }
                }
                return null
            }
        }
        val conf = ClientConfiguration()
        conf.connectionTimeout = 15 * 1000 // 连接超时，默认15秒
        conf.socketTimeout = 15 * 1000 // socket超时，默认15秒
        conf.maxConcurrentRequest = 5 // 最大并发请求书，默认5个
        conf.maxErrorRetry = 2 // 失败后最大重试次数，默认2次
        oss = OSSClient(context, OSS_ENDPOINT, credetialProvider, conf)
        if (BuildConfig.DEBUG) {
            OSSLog.enableLog()
        }
    }

    /**
     * 按图片路径上传
     *
     * @param type      拍照类型-vehicle_license_back
     * @param localFile /storage/emulated/0/Android/media/com.jime.stu/万能识图/2020-09-20-14-54-27-078.jpeg
     */
    fun uploadImg(type: String, localFile: String) {
        val file = File(localFile)
        if (!file.exists()) {
            ossUploadImgListener?.uploadImgFailForSubthread("路径不存在")
            return
        }

        // 构造上传请求
        var imgName = ""
        imgName = createImgName(type)
        val put = PutObjectRequest(OSS_BUCKET_NAME, imgName, localFile)
        asyncPutImage(put, imgName)
    }

    /**
     * 创建img全路径
     *
     * @return pkg_camera/img/20200921/2026418043601100000/xsz_back_1600677544590.jpeg
     */
    private fun createImgName(type: String): String {
        return OSS_ROOT_PATH +
                TimeRender.formatDate(Date(), TimeRender.DEFAULT_FORMAT_MM) +
                "/" + UserManager.getInstance().getUserID() +
                "/" + type + "_" + System.currentTimeMillis() +
                OSS_IMG_TYPE
    }

    /**
     * 异步图片上传
     *
     * @param put
     * @param imgName
     */
    private fun asyncPutImage(put: PutObjectRequest, imgName: String) {
        put.crC64 = OSSRequest.CRC64Config.YES
        task = oss?.asyncPutObject(
            put,
            object : OSSCompletedCallback<PutObjectRequest?, PutObjectResult?> {
                override fun onSuccess(request: PutObjectRequest?, result: PutObjectResult?) {
                    if (ossUploadImgListener != null) {
                        ossUploadImgListener!!.uploadImgSuccessForSubthread(OSS_NETWORK + imgName)
                    }
                }

                override fun onFailure(
                    request: PutObjectRequest?,
                    clientExcepion: ClientException,
                    serviceException: ServiceException
                ) {
                    var info = ""
                    if (clientExcepion != null) {
                        // 本地异常如网络异常等
                        info = clientExcepion.toString()
                    }
                    if (serviceException != null) {
                        // 服务异常
                        info = serviceException.toString()
                    }
                    if (ossUploadImgListener != null) {
                        ossUploadImgListener!!.uploadImgFailForSubthread(info)
                    }
                }
            })
    }

    /**
     * 取消上传任务
     */
    fun cancelUploadImgTask() {
        if (task != null && !task!!.isCompleted) {
            task!!.cancel()
        }
    }

    fun setOssUploadImgListener(ossUploadImgListener: OssUploadImgListener?) {
        this.ossUploadImgListener = ossUploadImgListener
    }

    interface OssUploadImgListener {
        /**
         * 上传失败 回调结果在子线程
         *
         * @param errorInfo 失败信息
         */
        fun uploadImgFailForSubthread(errorInfo: String)

        /**
         * 上传成功 回调结果在子线程
         *
         * @param imgPath oss图片路径
         */
        fun uploadImgSuccessForSubthread(imgPath: String?)
    }
}